library(survival)

source("R/getwhas100dataset.R")
source("R/SurvivalDataSet.R")
source("R/wod_4.R")
source("R/wod_6.R")
source("R/wod_coxdevianceresiduals.R")
source("R/wod_coxld.R")
source("R/wod_coxmartingaleresiduals.R")

source('R/bootstraps.R')
source("R/wod_4biasedhist.R")
source("R/wod_4biasedhist_k.R")
source("R/paod_t_onesided.R")

#' Extract the most outlying observations following a criteria based on concordance
#' @param surv.object An obect of type survival::Surv containing lifetimes and right-censoring status
#' @param covariate.data A data frame containing the data with covariate values for each individual
#' @param sod.method One of c("osd","bht","dbht","ld","martingale","deviance")
#' @param B The number of bootstrap samples generated only applicable for "bht" and "dbht" methods
#' @param B.N the number of observations in each bootstrap sample
#' @param max.outliers Ths parameter is used for the "osd" method   
#' @return A data.frame containing the most outlying observations sorted by outlying score    
#' 
#' @export
bcsod <- function(surv.object, covariate.data, sod.method, B , B.N , max.outliers ){
  
  if( ! is.Surv(surv.object) ){
    
    stop("Parameter \"surv.object\" must be an object of type \"survival::Surv\" ")
  }
  
  if( ! is.data.frame(covariate.data) ){
    
    stop("Parameter \"covariate.data\" must be of type \"data.frame\".")
  } 
  
  
  # obtain the number of records (individuals)
  N <- attr(surv.object,"dim")[1]
  
  t <- surv.object[1:N,1]
  s <- surv.object[1:N,2]
  # create a survival dataset from the Surv object and data
  surv_dataset <- SurvivalDataSet(data = covariate.data ,time_vector = t, status_vector = s)
  
  
  if( sod.method=="bht" ){
    
    outlier_set <- wod_4(dataset = surv_dataset ,nruns = B )
    return( outlier_set)
  }
  
  if( sod.method=="osd" ){
    outlier_set <- wod_6(dataset = surv_dataset, max_outliers = max.outliers )
    return( outlier_set)
  }
  
  if(sod.method=="dbht" ){
    antidotes_hist               <- wod_4hist_k(dataset = surv_dataset, nruns = B, m = B.N) 
    poisons_hist                 <- wod_4biasedhist_k(dataset = surv_dataset, nruns = B , m = B.N)  
    resulting_outliers           <- paod_t_onesided(poison_boot = poisons_hist , antidote_boot = antidotes_hist)

    return(resulting_outliers)
  }
  
  if( sod.method=="ld" ){
    outlier_set <- wod_coxld(dataset = surv_dataset)
    return(outlier_set)
    
  }
  
  if( sod.method=="martingale"){
    outlier_set <- wod_coxmartingaleresiduals(dataset = surv_dataset)
    return(outlier_set)
  }
  
  if( sod.method=="deviance"){
    outlier_set <- wod_coxdevianceresiduals(dataset = surv_dataset)
    return(outlier_set)
  }
  
  stop("Parameter \"sod.method\" must be one of c(\"osd\",\"bht\",\"dbht\",\"ld\",\"martingale\",\"deviance\") ")
  
}
